@@ -137,13 +137,15 @@ def parse_requirements(requirements_files=None, strip_markers=False):
         # such as:
         # -e git://github.com/openstack/nova/master#egg=nova
         # -e git://github.com/openstack/nova/master#egg=nova-1.2.3
+        # -e git+https://foo.com/zipball#egg=bar&subdirectory=baz
         if re.match(r'\s*-e\s+', line):
-            line = re.sub(r'\s*-e\s+.*#egg=(.*)$', egg_fragment, line)
+            line = re.sub(r'\s*-e\s+.*#egg=([^&]+).*$', egg_fragment, line)
         # such as:
         # http://github.com/openstack/nova/zipball/master#egg=nova
         # http://github.com/openstack/nova/zipball/master#egg=nova-1.2.3
+        # git+https://foo.com/zipball#egg=bar&subdirectory=baz
         elif re.match(r'\s*(https?|git(\+(https|ssh))?):', line):
-            line = re.sub(r'\s*(https?|git(\+(https|ssh))?):.*#egg=(.*)$',
+            line = re.sub(r'\s*(https?|git(\+(https|ssh))?):.*#egg=([^&]+).*$',
                           egg_fragment, line)
         # -f lines are for index locations, and don't get used here
         elif re.match(r'\s*-f\s+', line):
@@ -217,8 +219,14 @@ class TestrTest(testr_command.Testr):
     """Make setup.py test do the right thing."""
 
     command_name = 'test'
+    description = 'DEPRECATED: Run unit tests using testr'
 
     def run(self):
+        warnings.warn('testr integration is deprecated in pbr 4.2 and will '
+                      'be removed in a future release. Please call your test '
+                      'runner directly',
+                      DeprecationWarning)
+
         # Can't use super - base class old-style class
         testr_command.Testr.run(self)
 
@@ -304,8 +312,6 @@ if __name__ == "__main__":
     import sys
     import wsgiref.simple_server as wss
 
-    my_ip = socket.gethostbyname(socket.gethostname())
-
     parser = argparse.ArgumentParser(
         description=%(import_target)s.__doc__,
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
@@ -488,6 +494,15 @@ class LocalManifestMaker(egg_info.manifest_maker):
             self.filelist.process_template_line(template_line)
 
     def add_defaults(self):
+        """Add all the default files to self.filelist:
+
+        Extends the functionality provided by distutils to also included
+        additional sane defaults, such as the ``AUTHORS`` and ``ChangeLog``
+        files generated by *pbr*.
+
+        Warns if (``README`` or ``README.txt``) or ``setup.py`` are missing;
+        everything else is optional.
+        """
         option_dict = self.distribution.get_option_dict('pbr')
 
         sdist.sdist.add_defaults(self)
